"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollisionAggregator = void 0;
var line_line_intersector_1 = require("./line_line_intersector");
var aggregation_tree_1 = require("./tree/aggregation_tree");
var line_point_intersector_1 = require("./line_point_intersector");
var CollisionAggregator = /** @class */ (function () {
    function CollisionAggregator() {
        this._intersectorRegistry = [new line_line_intersector_1.LineLineIntersector(), new line_point_intersector_1.LinePointIntersector()];
    }
    // aggregate commands together if they intersect
    CollisionAggregator.prototype.aggregate = function (shapes) {
        var _this = this;
        var aggregationTree = new aggregation_tree_1.AggregationTree(function (shape1, shape2) { return _this.intersect(shape1, shape2, _this._intersectorRegistry); });
        shapes.forEach(function (shape) {
            aggregationTree.addValue(shape);
        });
        return aggregationTree.getAggregatedCommands();
    };
    // find intersector for two commands and call it
    CollisionAggregator.prototype.intersect = function (shape1, shape2, registry) {
        var matchingIntersector = registry.find(function (intersector) {
            var cmdType = intersector.supportedShapeTypes();
            return (shape1.code === cmdType[0] && shape2.code === cmdType[1]) ||
                (shape1.code === cmdType[1] && shape2.code === cmdType[0]);
        });
        if (matchingIntersector === undefined) {
            throw new Error("intersector not defined for types ".concat(shape1.code, " ").concat(shape2.code));
        }
        if (matchingIntersector.supportedShapeTypes()[0] === shape1.code) {
            return matchingIntersector.intersects(shape1, shape2);
        }
        else {
            return matchingIntersector.intersects(shape2, shape1);
        }
    };
    return CollisionAggregator;
}());
exports.CollisionAggregator = CollisionAggregator;
