"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LineSegmentIntersector = void 0;
var vector2d_1 = require("../../base/vector2d");
var LineSegmentIntersector = /** @class */ (function () {
    function LineSegmentIntersector() {
    }
    LineSegmentIntersector.prototype.intersects = function (cmd1, cmd2) {
        var p1 = new vector2d_1.Vector2d(cmd1.x, cmd1.y);
        var p2 = new vector2d_1.Vector2d(cmd1.x0, cmd1.y0);
        var q1 = new vector2d_1.Vector2d(cmd2.x, cmd2.y);
        var q2 = new vector2d_1.Vector2d(cmd2.x0, cmd2.y0);
        var r = p2.deduct(p1);
        var s = q2.deduct(q1);
        var rxs = r.crossProduct(s);
        var qpxr = q1.deduct(p1).crossProduct(r);
        // If r x s = 0 and (q - p) x r = 0, then the two lines are collinear.
        if (this.isZero(rxs) && this.isZero(qpxr)) {
            // 1. If either  0 <= (q - p) * r <= r * r or 0 <= (p - q) * s <= * s
            // then the two lines are overlapping,
            if ((0 <= q1.deduct(p1).multiplyV(r).length() && q1.deduct(p1).multiplyV(r).length() <= r.multiplyV(r).length()) ||
                0 <= (p1.deduct(q1).multiplyV(s).length() && p1.deduct(q1).multiplyV(s) <= s.multiplyV(s)))
                return true;
            // 2. If neither 0 <= (q - p) * r = r * r nor 0 <= (p - q) * s <= s * s
            // then the two lines are collinear but disjoint.
            // No need to implement this expression, as it follows from the expression above.
            return false;
        }
        // 3. If r x s = 0 and (q - p) x r != 0, then the two lines are parallel and non-intersecting.
        if (this.isZero(rxs) && !this.isZero(qpxr)) {
            return false;
        }
        // t = (q - p) x s / (r x s)
        var t = (q1.deduct(p1)).crossProduct(s) / rxs;
        // u = (q - p) x r / (r x s)
        var u = (q1.deduct(p1)).crossProduct(r) / rxs;
        // 4. If r x s != 0 and 0 <= t <= 1 and 0 <= u <= 1
        // the two line segments meet at the point p + t r = q + u s.
        if (!this.isZero(rxs) && (0 <= t && t <= 1) && (0 <= u && u <= 1)) {
            // An intersection was found.
            return true;
        }
        // 5. Otherwise, the two line segments are not parallel but do not intersect.
        return false;
    };
    LineSegmentIntersector.prototype.supportedShapeTypes = function () {
        return ["L", "L"];
    };
    LineSegmentIntersector.prototype.isZero = function (d) {
        return Math.abs(d) < 1e-10;
    };
    return LineSegmentIntersector;
}());
exports.LineSegmentIntersector = LineSegmentIntersector;
