"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PhysicBuilder = void 0;
var matterjs_physics_1 = require("../physics/matterjs_physics");
var svg_data_1 = require("../base/svg_data");
var command_mapper_1 = require("../base/command_mapper");
var svg_path_parser_1 = require("svg-path-parser");
var render_builder_1 = require("./render_builder");
var gravity_plugin_1 = require("../physics/plugins/gravity_plugin");
var view_collider_border_plugin_1 = require("../physics/plugins/view_collider_border_plugin");
var mouse_plugin_1 = require("../physics/plugins/mouse_plugin");
var PhysicBuilder = /** @class */ (function () {
    function PhysicBuilder(aggregator, svg) {
        this._aggregator = aggregator;
        this._svg = svg;
        this._plugins = [];
    }
    PhysicBuilder.prototype.withGravity = function (gravity) {
        this._plugins.push(new gravity_plugin_1.GravityPlugin(gravity));
        return this;
    };
    PhysicBuilder.prototype.withBorderCollider = function () {
        this._plugins.push(new view_collider_border_plugin_1.ViewColliderBorder());
        return this;
    };
    PhysicBuilder.prototype.withMouseInteraction = function (mouseForce) {
        this._plugins.push(new mouse_plugin_1.MousePlugin(mouseForce));
        return this;
    };
    PhysicBuilder.prototype.createSvgBodies = function (config) {
        var svgData = this.createSvgData();
        var physics = new matterjs_physics_1.MatterJsPhysics(svgData, config, this._plugins);
        return new render_builder_1.RenderBuilder(physics, svgData);
    };
    PhysicBuilder.prototype.createSvgData = function () {
        var paths = Array.prototype.slice.call(this._svg.getElementsByTagName('path'));
        var pathCommandArray = paths
            .map(function (path) { return path.getAttribute("d"); })
            .map(function (pathString) { return (0, command_mapper_1.expandCommands)((0, svg_path_parser_1.makeAbsolute)((0, svg_path_parser_1.parseSVG)(pathString))); });
        var aggregatedCommands = this._aggregator.aggregate(pathCommandArray);
        return new svg_data_1.SvgData(this._svg, pathCommandArray, aggregatedCommands);
    };
    return PhysicBuilder;
}());
exports.PhysicBuilder = PhysicBuilder;
