"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StarterBuilder = void 0;
var command_mapper_1 = require("../base/command_mapper");
var svg_path_parser_1 = require("svg-path-parser");
var svg_data_1 = require("../base/svg_data");
var debug_renderer_1 = require("../renderer/debug_renderer");
var production_renderer_1 = require("../renderer/production_renderer");
var StarterBuilder = /** @class */ (function () {
    function StarterBuilder(aggregatorBuilder) {
        this._aggregatorBuilder = aggregatorBuilder;
    }
    StarterBuilder.prototype.startDebug = function () {
        var svgData = this.createSvgData();
        var renderer = new debug_renderer_1.DebugRenderer();
        renderer.renderLoop(svgData);
    };
    StarterBuilder.prototype.startProduction = function () {
        var svgData = this.createSvgData();
        var renderer = new production_renderer_1.ProductionRenderer();
        renderer.renderLoop(svgData);
    };
    StarterBuilder.prototype.createSvgData = function () {
        var svg = this._aggregatorBuilder._svg;
        var paths = Array.prototype.slice.call(svg.getElementsByTagName('path'));
        var pathCommandArray = paths
            .map(function (path) { return path.getAttribute("d"); })
            .map(function (pathString) { return (0, command_mapper_1.expandCommands)((0, svg_path_parser_1.makeAbsolute)((0, svg_path_parser_1.parseSVG)(pathString))); });
        var aggregatedCommands = this._aggregatorBuilder._aggregator.aggregate(pathCommandArray);
        return new svg_data_1.SvgData(svg, pathCommandArray, aggregatedCommands);
    };
    return StarterBuilder;
}());
exports.StarterBuilder = StarterBuilder;
