"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformCommands = void 0;
/**
 * apply all transformation matrices to path coordinates
 */
var transformCommands = function (svg, path, commands) {
    var parents = [svg].concat(findParentsTillSvg(path));
    parents.forEach(function (parent) {
        var transform = getComputedStyle(parent).transform;
        if (transform !== null && transform !== "none") {
            var matrix_1 = new DOMMatrix(transform);
            commands.forEach(function (command) { return transformCommand(matrix_1, command); });
        }
    });
};
exports.transformCommands = transformCommands;
var transformCommand = function (matrix, command) {
    switch (command.code) {
        case "S":
            {
                var p1 = matrix.transformPoint(new DOMPoint(command.x, command.y));
                command.x = p1.x;
                command.y = p1.y;
                var p2 = matrix.transformPoint(new DOMPoint(command.x2, command.y2));
                command.x2 = p2.x;
                command.y2 = p2.y;
                break;
            }
        case "Q":
            {
                var p1 = matrix.transformPoint(new DOMPoint(command.x, command.y));
                command.x = p1.x;
                command.y = p1.y;
                var p2 = matrix.transformPoint(new DOMPoint(command.x1, command.y1));
                command.x1 = p2.x;
                command.y1 = p2.y;
                break;
            }
        case "C":
            {
                var p1 = matrix.transformPoint(new DOMPoint(command.x, command.y));
                command.x = p1.x;
                command.y = p1.y;
                var p2 = matrix.transformPoint(new DOMPoint(command.x1, command.y1));
                command.x1 = p2.x;
                command.y1 = p2.y;
                var p3 = matrix.transformPoint(new DOMPoint(command.x1, command.y1));
                command.x1 = p3.x;
                command.y1 = p3.y;
                break;
            }
        case "M":
            {
                var p1 = matrix.transformPoint(new DOMPoint(command.x, command.y));
                command.x = p1.x;
                command.y = p1.y;
                break;
            }
        case "H":
            {
                throw new Error("H command not supported");
            }
        case "V":
            {
                throw new Error("H command not supported");
            }
        case "A":
            {
                throw new Error("A command not supported");
            }
    }
};
var findParentsTillSvg = function (element) {
    var result = [];
    result.push(element);
    if (element.parentElement !== null && element.tagName.toLowerCase() !== 'svg') {
        return result.concat(findParentsTillSvg(element.parentElement));
    }
    return result;
};
