"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CurveShape = void 0;
var bezier_js_1 = require("bezier-js");
var CurveShape = /** @class */ (function () {
    function CurveShape(curve, svgData) {
        this.code = 'C';
        this._curve = curve;
        this._svgData = svgData;
    }
    Object.defineProperty(CurveShape.prototype, "command", {
        get: function () {
            return this._curve;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CurveShape.prototype, "svgData", {
        get: function () {
            return this._svgData;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CurveShape.prototype, "center", {
        get: function () {
            var mid = this.bezier.get(0.5);
            return { x: mid.x, y: mid.y };
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CurveShape.prototype, "curvePoints", {
        get: function () {
            switch (this.command.code) {
                case 'C':
                    {
                        return [
                            { x: this.command.x0, y: this.command.y0 },
                            { x: this.command.x1, y: this.command.y1 },
                            { x: this.command.x2, y: this.command.y2 },
                            { x: this.command.x, y: this.command.y },
                        ];
                    }
                case 'Q':
                    {
                        return [
                            { x: this.command.x0, y: this.command.y0 },
                            { x: this.command.x1, y: this.command.y1 },
                            { x: this.command.x, y: this.command.y },
                        ];
                    }
                case "S":
                    {
                        return [
                            { x: this.command.x0, y: this.command.y0 },
                            { x: this.command.x2, y: this.command.y2 },
                            { x: this.command.x, y: this.command.y },
                        ];
                    }
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CurveShape.prototype, "bezier", {
        get: function () {
            return new bezier_js_1.Bezier(this.curvePoints);
        },
        enumerable: false,
        configurable: true
    });
    return CurveShape;
}());
exports.CurveShape = CurveShape;
