"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniqueValueNode = void 0;
var UniqueValueNode = /** @class */ (function () {
    function UniqueValueNode(value, parent) {
        this._children = [];
        this._value = value;
        this._parent = parent;
    }
    Object.defineProperty(UniqueValueNode.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UniqueValueNode.prototype, "children", {
        get: function () {
            return this._children;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UniqueValueNode.prototype, "parent", {
        get: function () {
            return this._parent;
        },
        enumerable: false,
        configurable: true
    });
    UniqueValueNode.prototype.getOrCreateNode = function (value) {
        var node = this.getNodeByValueRecursive(value);
        if (node === undefined) {
            node = new UniqueValueNode(value, this);
            this._children.push(node);
        }
        return node;
    };
    UniqueValueNode.prototype.changeParent = function (parent, child) {
        var parentNode = this.getOrCreateNode(parent);
        var childNode = this.getOrCreateNode(child);
        // remove child from previous parent
        if (childNode._parent !== undefined) {
            childNode._parent._children.splice(childNode._parent._children.indexOf(childNode), 1);
        }
        childNode._parent = parentNode;
        parentNode._children.push(childNode);
    };
    UniqueValueNode.prototype.getValuesRecursively = function () {
        var result = new Array();
        result.push(this._value);
        var childValues = this._children.reduce(function (acc, cur) { return acc.concat(cur.getValuesRecursively()); }, new Array());
        return result.concat(childValues);
    };
    UniqueValueNode.prototype.getNodeByValueRecursive = function (value) {
        if (this.value === value)
            return this;
        return this._children
            .map(function (child) { return child.getNodeByValueRecursive(value); })
            .filter(function (node) { return node !== undefined; })[0];
    };
    return UniqueValueNode;
}());
exports.UniqueValueNode = UniqueValueNode;
