"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.CharacterPathParser = void 0;
var tree_1 = require("../base/tree");
var command_1 = require("./command");
var rectangle_1 = require("../base/rectangle");
var path_parser_1 = require("./path_parser");
var CharacterPathParser = /** @class */ (function (_super) {
    __extends(CharacterPathParser, _super);
    function CharacterPathParser(path) {
        var _this = _super.call(this) || this;
        var characterParts = path
            .replace(/(\r\n|\n|\r)/gm, "") // new lines
            .replace('\t', '') // tabs
            .replace(/\s\s+/g, ' ') // more than one white space
            .split(/(?=[A-Z]+|[a-z]+)/) // split by characters
            .map(function (cmdWithParams) {
            var cmdName = cmdWithParams.charAt(0);
            var cmdParameters = _this.parseParameters(cmdWithParams.substring(1, cmdWithParams.length));
            switch (cmdName) {
                case "M": return new command_1.SinglePositionCommand(cmdName, cmdParameters);
                case "L": return new command_1.SinglePositionCommand(cmdName, cmdParameters);
                case "Z": return new command_1.ClosePathCommand(cmdName, cmdParameters);
                case "Q": return new command_1.SimpleBezierCurveCommand(cmdName, cmdParameters);
                case "A": return new command_1.ArcCommand(cmdName, cmdParameters);
                case "C": return new command_1.BezierCurveCommand(cmdParameters);
                default: throw new Error("command ".concat(cmdName, " not implemented"));
            }
        })
            // aggregate commands by move command
            .reduce(function (acc, cmd) {
            if (cmd.name === 'M')
                acc.push([cmd]);
            else
                acc[acc.length - 1].push(cmd);
            return acc;
        }, new Array);
        _this._charsCommands = combineCharacterParts(characterParts);
        return _this;
    }
    CharacterPathParser.prototype.parseParameters = function (parameters) {
        if (parameters == "")
            return [];
        return parameters.trim().split(" ");
    };
    ;
    return CharacterPathParser;
}(path_parser_1.AbstractPathParser));
exports.CharacterPathParser = CharacterPathParser;
var combineCharacterParts = function (characterParts) {
    var tree = new tree_1.UniqueValueNode(-1, undefined);
    // check if Character parts overlap
    for (var i = 0; i < characterParts.length; i++) {
        tree.getOrCreateNode(i);
        for (var j = 0; j < characterParts.length; j++) {
            if (i === j)
                continue;
            var characterPart = characterParts[i].reduce(function (acc, cmd) { return acc.concat(cmd.positions); }, new Array());
            var characterPartToCheck = characterParts[j].reduce(function (acc, cmd) { return acc.concat(cmd.positions); }, new Array());
            var collided = doesCharacterPartsCollide(characterPart, characterPartToCheck);
            if (collided)
                tree.changeParent(i, j);
        }
    }
    return tree.children
        .map(function (child) { return child.getValuesRecursively(); })
        .map(function (characterPartIndexes) {
        return characterPartIndexes.reduce(function (acc, index) { return acc.concat(characterParts[index]); }, new Array);
    });
};
var doesCharacterPartsCollide = function (part, partContained) {
    var bb1 = (0, rectangle_1.createBoundingBox)(part);
    var bb2 = (0, rectangle_1.createBoundingBox)(partContained);
    return (0, rectangle_1.isOverlaping)(bb1, bb2);
};
