import { Vector2d } from "../base/vector2d";
export interface Command {
    get name(): String;
    get numberOfPositions(): number;
    get positions(): Array<Vector2d>;
    set positions(positions: Array<Vector2d>);
    serialize(): string;
}
declare abstract class AbstractCommand implements Command {
    private _name;
    private _parameters;
    constructor(name: String, parameters: string[]);
    abstract get numberOfPositions(): number;
    abstract get positions(): Vector2d[];
    abstract set positions(positions: Vector2d[]);
    get name(): String;
    serialize(): string;
    protected getParameter: (index: number) => number;
    protected setParameter: (position: Vector2d, startIndex: number) => void;
}
export declare class SinglePositionCommand extends AbstractCommand {
    get numberOfPositions(): number;
    get positions(): Vector2d[];
    set positions(positions: Vector2d[]);
}
export declare class ClosePathCommand extends AbstractCommand {
    get numberOfPositions(): number;
    get positions(): Vector2d[];
    set positions(positions: Vector2d[]);
    serialize(): string;
}
export declare class SimpleBezierCurveCommand extends AbstractCommand {
    get numberOfPositions(): number;
    get positions(): Vector2d[];
    set positions(positions: Vector2d[]);
}
export declare class BezierCurveCommand implements Command {
    private _positions;
    constructor(parameters: string[]);
    get positions(): Array<Vector2d>;
    get name(): String;
    get numberOfPositions(): number;
    serialize(): string;
    set positions(positions: Array<Vector2d>);
}
export declare class ArcCommand extends AbstractCommand {
    get numberOfPositions(): number;
    get positions(): Vector2d[];
    set positions(positions: Vector2d[]);
}
export {};
