"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArcCommand = exports.BezierCurveCommand = exports.SimpleBezierCurveCommand = exports.ClosePathCommand = exports.SinglePositionCommand = void 0;
var vector2d_1 = require("../base/vector2d");
var AbstractCommand = /** @class */ (function () {
    function AbstractCommand(name, parameters) {
        var _this = this;
        this.getParameter = function (index) { return _this._parameters[index]; };
        this.setParameter = function (position, startIndex) {
            _this._parameters[startIndex] = position.x;
            _this._parameters[startIndex + 1] = position.y;
        };
        this._name = name;
        this._parameters = parameters.map(function (p) { return parseFloat(p); });
    }
    Object.defineProperty(AbstractCommand.prototype, "name", {
        get: function () {
            return this._name;
        },
        enumerable: false,
        configurable: true
    });
    AbstractCommand.prototype.serialize = function () {
        return this._name + " " + this._parameters.map(function (p) { return p.toFixed(2); }).join(" ");
    };
    ;
    return AbstractCommand;
}());
var SinglePositionCommand = /** @class */ (function (_super) {
    __extends(SinglePositionCommand, _super);
    function SinglePositionCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SinglePositionCommand.prototype, "numberOfPositions", {
        get: function () {
            return 1;
        },
        enumerable: false,
        configurable: true
    });
    ;
    Object.defineProperty(SinglePositionCommand.prototype, "positions", {
        get: function () {
            return [new vector2d_1.Vector2d(this.getParameter(0), this.getParameter(1))];
        },
        set: function (positions) {
            this.setParameter(positions[0], 0);
        },
        enumerable: false,
        configurable: true
    });
    ;
    return SinglePositionCommand;
}(AbstractCommand));
exports.SinglePositionCommand = SinglePositionCommand;
var ClosePathCommand = /** @class */ (function (_super) {
    __extends(ClosePathCommand, _super);
    function ClosePathCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(ClosePathCommand.prototype, "numberOfPositions", {
        get: function () {
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    ;
    Object.defineProperty(ClosePathCommand.prototype, "positions", {
        get: function () {
            return new Array();
        },
        set: function (positions) {
        },
        enumerable: false,
        configurable: true
    });
    ClosePathCommand.prototype.serialize = function () {
        return this.name.toString();
    };
    ;
    return ClosePathCommand;
}(AbstractCommand));
exports.ClosePathCommand = ClosePathCommand;
var SimpleBezierCurveCommand = /** @class */ (function (_super) {
    __extends(SimpleBezierCurveCommand, _super);
    function SimpleBezierCurveCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SimpleBezierCurveCommand.prototype, "numberOfPositions", {
        get: function () {
            return 2;
        },
        enumerable: false,
        configurable: true
    });
    ;
    Object.defineProperty(SimpleBezierCurveCommand.prototype, "positions", {
        get: function () {
            return [
                new vector2d_1.Vector2d(this.getParameter(0), this.getParameter(1)),
                new vector2d_1.Vector2d(this.getParameter(2), this.getParameter(3))
            ];
        },
        set: function (positions) {
            this.setParameter(positions[0], 0);
            this.setParameter(positions[1], 2);
        },
        enumerable: false,
        configurable: true
    });
    ;
    ;
    return SimpleBezierCurveCommand;
}(AbstractCommand));
exports.SimpleBezierCurveCommand = SimpleBezierCurveCommand;
var BezierCurveCommand = /** @class */ (function () {
    function BezierCurveCommand(parameters) {
        this._positions = parameters
            .map(function (p) { return parseFloat(p); })
            .reduce(function (acc, cur, index, array) {
            if (index % 2 == 0) {
                acc.push(new vector2d_1.Vector2d(array[index], array[index + 1]));
            }
            return acc;
        }, new Array());
    }
    Object.defineProperty(BezierCurveCommand.prototype, "positions", {
        get: function () {
            return this._positions;
        },
        set: function (positions) {
            this._positions = positions;
        },
        enumerable: false,
        configurable: true
    });
    ;
    Object.defineProperty(BezierCurveCommand.prototype, "name", {
        get: function () {
            return "C";
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BezierCurveCommand.prototype, "numberOfPositions", {
        get: function () {
            return this._positions.length;
        },
        enumerable: false,
        configurable: true
    });
    BezierCurveCommand.prototype.serialize = function () {
        return this.name + " " + this._positions
            .map(function (p) { return p.x.toFixed(2) + " " + p.y.toFixed(2); })
            .join(" ");
    };
    ;
    ;
    return BezierCurveCommand;
}());
exports.BezierCurveCommand = BezierCurveCommand;
var ArcCommand = /** @class */ (function (_super) {
    __extends(ArcCommand, _super);
    function ArcCommand() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(ArcCommand.prototype, "numberOfPositions", {
        get: function () {
            return 1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ArcCommand.prototype, "positions", {
        get: function () {
            return [new vector2d_1.Vector2d(this.getParameter(5), this.getParameter(6))];
        },
        set: function (positions) {
            this.setParameter(positions[0], 5);
        },
        enumerable: false,
        configurable: true
    });
    ;
    return ArcCommand;
}(AbstractCommand));
exports.ArcCommand = ArcCommand;
