"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractCurveBody = void 0;
var bezier_1 = require("../../bezierjs/bezier");
var matter_js_1 = require("matter-js");
var math_utils_1 = require("../../base/math_utils");
var command_utils_1 = require("../../base/command_utils");
var AbstractCurveBody = /** @class */ (function () {
    function AbstractCurveBody() {
    }
    AbstractCurveBody.prototype.createPoints = function (bezier, numberOfPoints) {
        var fraction = 1 / numberOfPoints;
        return Array.from(Array(numberOfPoints + 1).keys()).map(function (i) { return bezier.get(i * fraction); });
    };
    AbstractCurveBody.prototype.createInnerBodies = function (colliderSize, curve) {
        var curvePoints = (0, command_utils_1.findCurvePoints)(curve);
        var bezierPoints = this.createPoints(new bezier_1.Bezier(curvePoints), 3);
        return bezierPoints
            .filter(function (bp, index) { return index < bezierPoints.length - 1; })
            .map(function (bp, index) {
            var current = matter_js_1.Vector.create(bp.x, bp.y);
            var next = matter_js_1.Vector.create(bezierPoints[index + 1].x, bezierPoints[index + 1].y);
            var direction = matter_js_1.Vector.normalise(matter_js_1.Vector.sub(next, current));
            var midPoint = matter_js_1.Vector.create((current.x + next.x) / 2, (current.y + next.y) / 2);
            var d = (0, math_utils_1.findDistanceVec)(next, current);
            return matter_js_1.Bodies.rectangle(midPoint.x, midPoint.y, d, colliderSize, {
                angle: (0, math_utils_1.findAngle)(next.x, next.y, current.x, current.y)
            });
        });
    };
    return AbstractCurveBody;
}());
exports.AbstractCurveBody = AbstractCurveBody;
