"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MatterJsPhysics = void 0;
var Matter = require("matter-js");
var abstract_body_1 = require("./body/abstract_body");
var Engine = Matter.Engine;
var Runner = Matter.Runner;
var Bodies = Matter.Bodies;
var MatterJsPhysics = /** @class */ (function () {
    function MatterJsPhysics() {
        this._engine = Engine.create();
        this._world = this._engine.world;
        this._bodies = [];
    }
    MatterJsPhysics.prototype.init = function (aggregatedShapes) {
        var _this = this;
        aggregatedShapes.forEach(function (shapes) {
            _this._bodies = shapes.map(function (shape) { return (0, abstract_body_1.physBodyFactory)(shape, _this._world); });
            var midX = shapes
                .map(function (shape) { return shape.center.x; })
                .reduce(function (acc, cur) { return acc + cur; }) / shapes.length;
            var midY = shapes
                .map(function (shape) { return shape.center.y; })
                .reduce(function (acc, cur) { return acc + cur; }) / shapes.length;
            var aggregatedShape = Bodies.circle(midX, midY, 1);
            aggregatedShape.parts = [aggregatedShape].concat(_this._bodies.map(function (physBody) { return physBody.body; }));
        });
    };
    MatterJsPhysics.prototype.update = function (aggregatedShapes, deltaTime) {
        // update physics
        Engine.update(this._engine);
        this._bodies.forEach(function (body) { return body.setPositionsToShape(); });
    };
    return MatterJsPhysics;
}());
exports.MatterJsPhysics = MatterJsPhysics;
