"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MatterJsPhysics = void 0;
var Matter = require("matter-js");
var matter_js_1 = require("matter-js");
var command_utils_1 = require("../base/command_utils");
var body_factory_1 = require("./body/body_factory");
var Engine = Matter.Engine;
var Runner = Matter.Runner;
var Bodies = Matter.Bodies;
var MatterJsPhysics = /** @class */ (function () {
    function MatterJsPhysics(svgData, globalConfig, plugins) {
        var _this = this;
        this._svgData = svgData;
        this._engine = Engine.create();
        this._world = this._engine.world;
        this._commandBodies = [];
        var rootBodies = [];
        svgData.aggregatedCommands.forEach(function (commands) {
            var midX = commands
                .map(function (command) { return (0, command_utils_1.findCenterOfCommand)(command).x; })
                .reduce(function (acc, cur) { return acc + cur; }) / commands.length;
            var midY = commands
                .map(function (command) { return (0, command_utils_1.findCenterOfCommand)(command).y; })
                .reduce(function (acc, cur) { return acc + cur; }) / commands.length;
            var rootBody = Bodies.circle(midX, midY, 3000, globalConfig);
            var commandBodies = commands.map(function (command) { return (0, body_factory_1.physBodyFactory)(command, globalConfig.colliderWidth); });
            _this._commandBodies = _this._commandBodies.concat(commandBodies);
            var physBodies = commandBodies.map(function (physBody) { return physBody.bodies; }).flatMap(function (body) { return body; });
            matter_js_1.Body.setParts(rootBody, physBodies, false);
            matter_js_1.World.add(_this._world, rootBody);
            rootBodies.push(rootBody);
        });
        plugins.forEach(function (plugin) { return plugin.setup(svgData, _this._engine, rootBodies); });
    }
    MatterJsPhysics.prototype.update = function (aggregatedShapes) {
        // update physics
        Engine.update(this._engine);
        this._commandBodies.forEach(function (body) { return body.updateSvgCommand(); });
    };
    return MatterJsPhysics;
}());
exports.MatterJsPhysics = MatterJsPhysics;
