"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PivotSimulation = void 0;
var vector2d_1 = require("../base/vector2d");
var simulation_1 = require("./simulation");
var debug_1 = require("../debug");
var mouse_1 = require("./mouse");
var PivotSimulation = /** @class */ (function (_super) {
    __extends(PivotSimulation, _super);
    function PivotSimulation(svg) {
        var _this = _super.call(this, function () {
            return { pivot: undefined, angleAcceleration: 0 };
        }) || this;
        _this.findCenterOfMass = function (points) {
            var total = points.reduce(function (acc, cur) {
                acc.x += cur.x;
                acc.y += cur.y;
                return acc;
            }, { x: 0, y: 0 });
            return new vector2d_1.Vector2d(total.x / points.length, total.y / points.length);
        };
        _this._debugUI = new debug_1.DebugUI(svg);
        _this._mousePos = new mouse_1.RelativeMousePosition(svg);
        return _this;
    }
    PivotSimulation.prototype.simulateWithData = function (data, frameDuration, points) {
        // initialize pivot only first time
        if (data.pivot === undefined) {
            data.pivot = this.findPivot(points);
        }
        var centerOfMass = this.findCenterOfMass(points);
        // mouseForce
        var relativeMousePos = this._mousePos.position;
        var mouseForceVector = centerOfMass.deduct(relativeMousePos).opposite();
        var f = mouseForceVector.unit().multiply(10 / mouseForceVector.length()).opposite();
        data.angleAcceleration += this.calculateTangentForce(data.pivot, centerOfMass, f);
        // gravity
        var gravity = new vector2d_1.Vector2d(0, simulation_1.GRAVITY);
        data.angleAcceleration += this.calculateTangentForce(data.pivot, centerOfMass, gravity);
        // friction
        data.angleAcceleration *= 0.98;
        // special case
        if (isNaN(data.angleAcceleration)) {
            data.angleAcceleration = 0;
        }
        return points.map(function (point) {
            var rotationAngle = data.angleAcceleration * (Math.PI / 180) * (frameDuration / simulation_1.SIMULATION_SPEED);
            return point.rotateAroundPivot(data.pivot, rotationAngle);
        });
    };
    PivotSimulation.prototype.calculateTangentForce = function (pivot, centerOfMass, force) {
        var forcePosition = centerOfMass.add(force);
        var cancelledForcePosition = centerOfMass.deduct(pivot).add(centerOfMass);
        var angle = (0, vector2d_1.findAngle)(forcePosition, centerOfMass, cancelledForcePosition);
        // this._debugUI.drawLine(centerOfMass, centerOfMass.add(force.multiply(100)), "blue");
        // this._debugUI.drawLine(centerOfMass, cancelledForcePosition, "green");
        if (forcePosition.isAboveLine(cancelledForcePosition, pivot)) {
            return Math.sin(angle) * force.length();
        }
        else {
            return -Math.sin(angle) * force.length();
        }
    };
    PivotSimulation.prototype.findPivot = function (points) {
        var centerOfMass = this.findCenterOfMass(points);
        var mostTopPoint = __spreadArray([], points, true).sort(function (p1, p2) { return p1.y - p2.y; })[0];
        return new vector2d_1.Vector2d(centerOfMass.x, mostTopPoint.y);
    };
    return PivotSimulation;
}(simulation_1.AbstractSimulation));
exports.PivotSimulation = PivotSimulation;
