"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DebugRenderer = void 0;
var matterjs_physics_1 = require("../physics/matterjs_physics");
var DebugRenderer = /** @class */ (function () {
    function DebugRenderer(svgData, pathCommands, aggregatedCommands) {
        this._prevTime = 0;
        this._pathCommands = pathCommands;
        this._aggregatedCommands = aggregatedCommands;
        this._physics = new matterjs_physics_1.MatterJsPhysics(svgData, aggregatedCommands);
        var svgClone = svgData.svg.cloneNode(true);
        svgClone.id = svgClone.id + "1";
        this._svgClone = svgClone;
        svgData.svg.parentElement.append(svgClone);
        this._svgPaths = Array.prototype.slice.call(svgClone.getElementsByTagName('path'));
    }
    DebugRenderer.prototype.render = function (time) {
        var _this = this;
        this._prevTime = time;
        this._physics.update(this._aggregatedCommands, time);
        this._pathCommands.forEach(function (commands, index) {
            var pathString = commands.map(serializeCommand).join(" ");
            _this._svgPaths[index].setAttribute("d", pathString);
        });
        window.requestAnimationFrame(function (t) { return _this.render(t); });
    };
    return DebugRenderer;
}());
exports.DebugRenderer = DebugRenderer;
var serializeCommand = function (command) {
    switch (command.code) {
        case 'M':
            {
                var move = command;
                return "M ".concat(move.x.toFixed(2), " ").concat(move.y.toFixed(2));
            }
        case 'L':
            {
                var line = command;
                return "L ".concat(line.x.toFixed(2), " ").concat(line.y.toFixed(2));
            }
        case 'Z':
            {
                return "Z";
            }
        case 'C':
            {
                var curve = command;
                return "C ".concat(curve.x1.toFixed(2), " ").concat(curve.y1.toFixed(2), " ").concat(curve.x2.toFixed(2), " ").concat(curve.y2.toFixed(2), "  ").concat(curve.x.toFixed(2), " ").concat(curve.y.toFixed(2));
            }
        case 'S':
            {
                var curve = command;
                return "S ".concat(curve.x2.toFixed(2), " ").concat(curve.y2.toFixed(2), "  ").concat(curve.x.toFixed(2), " ").concat(curve.y.toFixed(2));
            }
        case 'Q':
            {
                var curve = command;
                return "Q ".concat(curve.x1.toFixed(2), " ").concat(curve.y1.toFixed(2), " ").concat(curve.x.toFixed(2), " ").concat(curve.y.toFixed(2));
            }
        default:
            {
                throw new Error("no serializer found for ".concat(command.code));
            }
    }
};
