"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LineLineIntersector = void 0;
var matter_js_1 = require("matter-js");
var math_utils_1 = require("../../base/math_utils");
var LineLineIntersector = /** @class */ (function () {
    function LineLineIntersector() {
    }
    LineLineIntersector.prototype.intersects = function (line1, line2) {
        var p1 = matter_js_1.Vector.create(line1.x, line1.y);
        var p2 = matter_js_1.Vector.create(line1.x0, line1.y0);
        var q1 = matter_js_1.Vector.create(line2.x, line2.y);
        var q2 = matter_js_1.Vector.create(line2.x0, line2.y0);
        var r = matter_js_1.Vector.sub(p2, p1);
        var s = matter_js_1.Vector.sub(q2, q1);
        var rxs = matter_js_1.Vector.cross(r, s);
        var qpxr = matter_js_1.Vector.cross(matter_js_1.Vector.sub(q1, p1), r);
        // If r x s = 0 and (q - p) x r = 0, then the two lines are collinear.
        if (this.isZero(rxs) && this.isZero(qpxr)) {
            var q1p1rLength = matter_js_1.Vector.magnitude((0, math_utils_1.multiplyVec)(matter_js_1.Vector.sub(q1, p1), r));
            var p1q1sLength = matter_js_1.Vector.magnitude((0, math_utils_1.multiplyVec)(matter_js_1.Vector.sub(p1, q1), s));
            var rSquaredLength = matter_js_1.Vector.magnitude((0, math_utils_1.multiplyVec)(r, r));
            var sSquaredLength = matter_js_1.Vector.magnitude((0, math_utils_1.multiplyVec)(s, s));
            // 1. If either  0 <= (q - p) * r <= r * r or 0 <= (p - q) * s <= * s
            // then the two lines are overlapping,
            if ((0 <= q1p1rLength && q1p1rLength <= rSquaredLength) ||
                0 <= (p1q1sLength && p1q1sLength <= sSquaredLength)) {
                return true;
            }
            // 2. If neither 0 <= (q - p) * r = r * r nor 0 <= (p - q) * s <= s * s
            // then the two lines are collinear but disjoint.
            // No need to implement this expression, as it follows from the expression above.
            return false;
        }
        // 3. If r x s = 0 and (q - p) x r != 0, then the two lines are parallel and non-intersecting.
        if (this.isZero(rxs) && !this.isZero(qpxr)) {
            return false;
        }
        // t = (q - p) x s / (r x s)
        var t = matter_js_1.Vector.cross(matter_js_1.Vector.sub(q1, p1), s) / rxs;
        // u = (q - p) x r / (r x s)
        var u = matter_js_1.Vector.cross(matter_js_1.Vector.sub(q1, p1), r) / rxs;
        // 4. If r x s != 0 and 0 <= t <= 1 and 0 <= u <= 1
        // the two line segments meet at the point p + t r = q + u s.
        if (!this.isZero(rxs) && (0 <= t && t <= 1) && (0 <= u && u <= 1)) {
            // An intersection was found.
            return true;
        }
        // 5. Otherwise, the two line segments are not parallel but do not intersect.
        return false;
    };
    LineLineIntersector.prototype.supportedCommandTypes = function () {
        return [["L", "Z"], ["L", "Z"]];
    };
    LineLineIntersector.prototype.isZero = function (d) {
        return Math.abs(d) < 1e-10;
    };
    return LineLineIntersector;
}());
exports.LineLineIntersector = LineLineIntersector;
