"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregationTree = void 0;
var aggregation_node_1 = require("./aggregation_node");
var AggregationTree = /** @class */ (function () {
    function AggregationTree(aggregationFcn) {
        this._roots = [];
        this._aggregationFcn = aggregationFcn;
    }
    AggregationTree.prototype.getAggregatedCommands = function () {
        return this._roots.map(function (root) {
            return root.getChildrenRecursively().map(function (node) { return node.value; });
        });
    };
    AggregationTree.prototype.addValue = function (value) {
        var _this = this;
        if (this._roots.length === 0) {
            this._roots.push(new aggregation_node_1.AggregationNode(value, undefined));
        }
        else {
            var nodesCollided = this.getAllNodes()
                .filter(function (node) { return _this._aggregationFcn(node.value, value); });
            // did not collide with anything
            if (nodesCollided.length === 0) {
                // add as root
                this._roots.push(new aggregation_node_1.AggregationNode(value, undefined));
            }
            else {
                var addedNode_1 = nodesCollided[0].addChild(value);
                nodesCollided.slice(1, nodesCollided.length)
                    .map(function (node) { return node.getRoot(); })
                    .filter(function (root) { return root.id !== addedNode_1.getRoot().id; })
                    .filter(function (node, index, self) {
                    return index === self.findIndex(function (t) { return (t.id === node.id); });
                })
                    .forEach(function (root) {
                    addedNode_1.addChildNode(root);
                    _this._roots.splice(_this._roots.indexOf(root), 1);
                });
            }
        }
    };
    AggregationTree.prototype.getAllNodes = function () {
        return this._roots.flatMap(function (node) { return node.getChildrenRecursively(); });
    };
    return AggregationTree;
}());
exports.AggregationTree = AggregationTree;
