"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CollisionAggregator = void 0;
var line_line_intersector_1 = require("./collision/line_line_intersector");
var aggregation_tree_1 = require("./collision/tree/aggregation_tree");
var line_point_intersector_1 = require("./collision/line_point_intersector");
var line_curve_intersector_1 = require("./collision/line_curve_intersector");
var curve_curve_intersector_1 = require("./collision/curve_curve_intersector");
var curve_point_intersector_1 = require("./collision/curve_point_intersector");
var point_point_intersector_1 = require("./collision/point_point_intersector");
var CollisionAggregator = /** @class */ (function () {
    function CollisionAggregator() {
        this._intersectorRegistry = [new line_line_intersector_1.LineLineIntersector(), new line_point_intersector_1.LinePointIntersector(),
            new line_curve_intersector_1.LineCurveIntersector(), new curve_curve_intersector_1.CurveCurveIntersector(), new curve_point_intersector_1.CurvePointIntersector(), new point_point_intersector_1.PointPointIntersector()];
    }
    // aggregate commands together if they intersect
    CollisionAggregator.prototype.aggregate = function (pathCommands) {
        var _this = this;
        var flatCommands = pathCommands
            .flatMap(function (commands) { return commands; });
        var aggregationTree = new aggregation_tree_1.AggregationTree(function (shape1, shape2) { return _this.intersect(shape1, shape2, _this._intersectorRegistry); });
        flatCommands.forEach(function (command) {
            aggregationTree.addValue(command);
        });
        return aggregationTree.getAggregatedCommands();
    };
    // find intersector for two commands and call it
    CollisionAggregator.prototype.intersect = function (cmd1, cmd2, registry) {
        var matchingIntersector = registry.find(function (intersector) {
            var cmdType = intersector.supportedCommandTypes();
            return (cmdType[0].includes(cmd1.code) && cmdType[1].includes(cmd2.code)) ||
                (cmdType[1].includes(cmd1.code) && cmdType[0].includes(cmd2.code));
        });
        if (matchingIntersector === undefined) {
            throw new Error("intersector not defined for types ".concat(cmd1.code, " ").concat(cmd2.code));
        }
        if (matchingIntersector.supportedCommandTypes()[0].includes(cmd1.code)) {
            return matchingIntersector.intersects(cmd1, cmd2);
        }
        else {
            return matchingIntersector.intersects(cmd2, cmd1);
        }
    };
    return CollisionAggregator;
}());
exports.CollisionAggregator = CollisionAggregator;
