"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findCurvePoints = exports.findLengthOfLineCommand = exports.findAngleOfLineCommand = exports.findCenterOfLineCommand = exports.findCenterOfCurveCommand = exports.findCenterOfCommand = void 0;
var matter_js_1 = require("matter-js");
var math_utils_1 = require("./math_utils");
var bezier_js_1 = require("../../bezierjs/bezier.js");
var findCenterOfCommand = function (command) {
    switch (command.code) {
        case "M": return matter_js_1.Vector.create(command.x, command.y);
        case "T": throw new Error("T not supported");
        case "Z": return (0, exports.findCenterOfLineCommand)(command);
        case "C": return (0, exports.findCenterOfCurveCommand)(command);
        case "Q": return (0, exports.findCenterOfCurveCommand)(command);
        case "S": return (0, exports.findCenterOfCurveCommand)(command);
        case "L": return (0, exports.findCenterOfLineCommand)(command);
    }
};
exports.findCenterOfCommand = findCenterOfCommand;
var findCenterOfCurveCommand = function (command) {
    var points = (0, exports.findCurvePoints)(command);
    var bezierCenter = new bezier_js_1.Bezier(points).get(0.5);
    return matter_js_1.Vector.create(bezierCenter.x, bezierCenter.y);
};
exports.findCenterOfCurveCommand = findCenterOfCurveCommand;
var findCenterOfLineCommand = function (command) {
    return (0, math_utils_1.findCenterOfLine)(matter_js_1.Vector.create(command.x, command.y), matter_js_1.Vector.create(command.x0, command.y0));
};
exports.findCenterOfLineCommand = findCenterOfLineCommand;
var findAngleOfLineCommand = function (command) {
    return (0, math_utils_1.findAngle)(command.x, command.y, command.x0, command.y0);
};
exports.findAngleOfLineCommand = findAngleOfLineCommand;
var findLengthOfLineCommand = function (command) {
    return (0, math_utils_1.findDistanceVec)(matter_js_1.Vector.create(command.x, command.y), matter_js_1.Vector.create(command.x0, command.y0));
};
exports.findLengthOfLineCommand = findLengthOfLineCommand;
var findCurvePoints = function (command) {
    switch (command.code) {
        case 'C':
            {
                return [
                    matter_js_1.Vector.create(command.x0, command.y0),
                    matter_js_1.Vector.create(command.x1, command.y1),
                    matter_js_1.Vector.create(command.x2, command.y2),
                    matter_js_1.Vector.create(command.x, command.y),
                ];
            }
        case 'Q':
            {
                return [
                    matter_js_1.Vector.create(command.x0, command.y0),
                    matter_js_1.Vector.create(command.x1, command.y1),
                    matter_js_1.Vector.create(command.x, command.y),
                ];
            }
        case "S":
            {
                return [
                    matter_js_1.Vector.create(command.x0, command.y0),
                    matter_js_1.Vector.create(command.x2, command.y2),
                    matter_js_1.Vector.create(command.x, command.y),
                ];
            }
        case "T":
            {
                throw new Error("T not defined");
            }
    }
};
exports.findCurvePoints = findCurvePoints;
